from sentential.support.shaper import Shaper, List, Field

"""
# Define, describe, and validate environment variables and build arguments.

# See documentation for general usage.

# Example:
class Env(Shaper):
    db_url: str
    db_port: int = 3306
    db_username: str = "postgres"
    db_password: str

#
# Compatible Classes
#

class Arg(Shaper):
    pass

class Env(Shaper):
    pass
"""
